@@ -14,17 +14,8 @@ import sphinx.util.matching
 import sphinx.util.docutils
 import sphinx.writers.html5
 
-from . import apidoc_formatting
-from . import autodoc_property_type
-from . import cpp_domain_fixes
-from . import inlinesyntaxhighlight
-from . import generic_synopses
+from .apidoc import apidoc_formatting
 from . import nav_adapt
-from . import object_toc
-from . import postprocess_html
-from . import python_domain_fixes
-from . import python_type_annotation_transforms
-from . import search_adapt
 from .details_patch import monkey_patch_details_run
 
 logger = sphinx.util.logging.getLogger(__name__)
@@ -135,6 +126,7 @@ def _get_html_builder(base_builder: Type[sphinx.builders.html.StandaloneHTMLBuil
             )
             if nav_adapt.READTHEDOCS is None:
                 excluded_scripts.add("_static/jquery.js")
+                excluded_scripts.add("_static/_sphinx_javascript_frameworks_compat.js")
             self.script_files = [
                 x for x in self.script_files if x.filename not in excluded_scripts
             ]
@@ -191,6 +183,9 @@ def _get_html_builder(base_builder: Type[sphinx.builders.html.StandaloneHTMLBuil
                         ]
                         if nav_adapt.READTHEDOCS is None:
                             excluded_list.append("**/jquery*.js")
+                            excluded_list.append(
+                                "**/_sphinx_javascript_frameworks_compat.js"
+                            )
                         excluded = sphinx.util.matching.Matcher(excluded_list)
                     else:
                         excluded = sphinx.util.matching.DOTFILES
@@ -310,16 +305,21 @@ def _config_inited(
 
 def setup(app):
     app.connect("config-inited", _config_inited)
+
     app.setup_extension(apidoc_formatting.__name__)
-    app.setup_extension(python_domain_fixes.__name__)
-    app.setup_extension(python_type_annotation_transforms.__name__)
-    app.setup_extension(cpp_domain_fixes.__name__)
+    app.setup_extension("sphinx_immaterial.apidoc.python.domain_fixes")
+    app.setup_extension("sphinx_immaterial.apidoc.python.type_annotation_transforms")
+    app.setup_extension("sphinx_immaterial.apidoc.cpp.domain_fixes")
     app.setup_extension(nav_adapt.__name__)
-    app.setup_extension(postprocess_html.__name__)
-    app.setup_extension(inlinesyntaxhighlight.__name__)
-    app.setup_extension(object_toc.__name__)
-    app.setup_extension(search_adapt.__name__)
-    app.setup_extension(generic_synopses.__name__)
+    app.setup_extension("sphinx_immaterial.postprocess_html")
+
+    if sphinx.version_info < (5, 0):
+        app.setup_extension("sphinx_immaterial.inlinesyntaxhighlight")
+
+    app.setup_extension("sphinx_immaterial.apidoc.object_toc")
+    app.setup_extension("sphinx_immaterial.search_adapt")
+    app.setup_extension("sphinx_immaterial.apidoc.generic_synopses")
+
     app.connect("html-page-context", html_page_context)
     app.connect("builder-inited", _builder_inited)
     app.add_config_value(
